﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=9.5
@EndOfDesignText@
' clsCard - My Standard Class
' Author: LucaMs
#Region VERSIONS 
' V. 1.0.0	01/30/2020
#End Region

#Region EVENTS' DECLARATIONS 
#Event: OneClick
#Event: DoubleOrLongClick
#Event: Touch(Action As Int, Coordinate() As Tp_Coordinate)
#End Region

Sub Class_Globals
	Private mCallback As Object
	Private mEventName As String
	
#Region PROPERTIES' MEMBER VARIABLES 
	Private mLeft, mTop As Int
	Private mWidth, mHeight As Int
	Private mBack, mFace, mCurrSide As B4XView
	Private mAngle As Int
	Private mAngleIncr As Float
	Private mTag As Object
	Private mEnabled As Boolean
	Private mAnimEnabled As Boolean
#End Region

	Public Const ANGLEINCR_10 As Float = 10
	Public Const ANGLEINCR_5 As Float = 5
	Public Const ANGLEINCR_2_5 As Float = 2.5
	Public Const ANGLEINCR_1 As Float = 1
	
	Private mB4XImageView As B4XImageView
	Private mFlipping As Boolean
End Sub

' Face: you can pass Null
Public Sub Initialize(Callback As Object, EventName As String, Back As B4XView, Face As B4XView)
	mCallback = Callback
	mEventName = EventName

	mBack = Back
	mFace = Face
	mCurrSide = mBack

	mB4XImageView.Initialize(Me, "mB4XImageView")
	
	mEnabled = True
	mAnimEnabled = True
	mAngleIncr = ANGLEINCR_10
End Sub

#Region PROPERTIES 

Public Sub setLeft(Left As Int)
	mLeft = Left
End Sub
Public Sub getLeft As Int
	Return mLeft
End Sub

Public Sub setTop(Top As Int)
	mTop = Top
End Sub
Public Sub getTop As Int
	Return mTop
End Sub

Public Sub setWidth(Width As Int)
	mWidth = Width
End Sub
Public Sub getWidth As Int
	Return mWidth
End Sub

Public Sub setHeight(Height As Int)
	mHeight = Height
End Sub
Public Sub getHeight As Int
	Return mHeight
End Sub

Public Sub setBack(Back As B4XView)
	mBack = Back
End Sub
Public Sub getBack As B4XView
	Return mBack
End Sub

Public Sub setFace(Face As B4XView)
	mFace = Face
End Sub
Public Sub getFace As B4XView
	Return mFace
End Sub

Public Sub setAngle(Angle As Int)
	mAngle = Angle
End Sub
Public Sub getAngle As Int
	Return mAngle
End Sub

Public Sub setTag(Tag As Object)
	mTag = Tag
End Sub
Public Sub getTag As Object
	Return mTag
End Sub

Public Sub setEnabled(Enabled As Boolean)
	mEnabled = Enabled
End Sub
Public Sub getEnabled As Boolean
	Return mEnabled
End Sub

Public Sub setAnimEnabled(Enabled As Boolean)
	mAnimEnabled = Enabled
End Sub
Public Sub getAnimEnabled As Boolean
	Return mAnimEnabled
End Sub


' Pass one of the "ANGLEINCR" constants provided by this class.
Public Sub setAngleIncrement(Incr As Float)
	If Incr <> ANGLEINCR_10 And _
		Incr <> ANGLEINCR_5 And _
		Incr <> ANGLEINCR_2_5 And _
		Incr <> ANGLEINCR_1 Then
		LogColor("Error setting the angle increment", Colors.Red)
		Return
	End If
	mAngleIncr = Incr
End Sub
Public Sub getAngleIncrement As Float
	Return mAngleIncr
End Sub

#End Region

#Region PUBLIC METHODS 

Public Sub AddToParent(Parent As B4XView, Left As Int, Top As Int, Width As Int, Height As Int)
	mB4XImageView.AddToParent(Parent, Left, Top, Width, Height)
	mB4XImageView.B4XBitmap = mBack
End Sub

Public Sub FlipAnim(Horizontally As Boolean)
	If Not(mAnimEnabled) Or mFlipping Then Return
	
	mFlipping = True
	
	For ang = 0 To 180 Step mAngleIncr
		mAngle = mAngle + mAngleIncr
		
		'Rotate the card to the newly set angle
		If Horizontally Then
			mB4XImageView.Rotate(0, mAngle, 0) 
		Else
			mB4XImageView.Rotate(mAngle, 0, 0)
		End If
		
		If ang = 90 Then
			If mCurrSide = mBack Then
				mCurrSide = mFace
			Else
				mCurrSide = mBack
			End If
			mB4XImageView.B4XBitmap = mCurrSide
			mAngle = 270
		End If
		Sleep(0)
	Next
	
	mFlipping = False
End Sub

Public Sub Show(Face As Boolean)
	If Face Then
		mB4XImageView.B4XBitmap = mFace 'Set card face image
	Else
		mB4XImageView.B4XBitmap = mBack 'Set card back image
	End If
End Sub

Public Sub GetXImageView As B4XImageView
	Return mB4XImageView
End Sub

#End Region

#Region PRIVATE METHODS 

Private Sub RaiseEvent(EvName As String, Params As List)
	Dim FullRoutineName As String
	FullRoutineName = mEventName & "_" & EvName
	If SubExists(mCallback, FullRoutineName) Then
		If Not(Params.IsInitialized) Or Params.Size = 0 Then
			CallSubDelayed(mCallback, FullRoutineName)
		Else
			Select Params.Size
				Case 1
					CallSubDelayed2(mCallback, FullRoutineName, Params.Get(0))
				Case 2
					CallSubDelayed3(mCallback, FullRoutineName, Params.Get(0), Params.Get(1))
			End Select
		End If
	End If
End Sub

#End Region

#Region EVENTS 

Private Sub mB4XImageView_OneClick
	If Not(mEnabled) Then Return
	RaiseEvent("OneClick", Null)
End Sub

Private Sub mB4XImageView_DoubleOrLongClick
	If Not(mEnabled) Then Return
	RaiseEvent("DoubleOrLongClick", Null)
End Sub

Private Sub mB4XImageView_Touch(Action As Int, Coordinate() As Tp_Coordinate)
	If Not(mEnabled) Then Return
	Dim lstParams As List
	lstParams.Initialize
	lstParams.AddAll(Array As Object(Action, Coordinate))
	RaiseEvent("Touch", lstParams)
End Sub

#End Region
